@@ -10,6 +10,7 @@
 #include <config.h>
 #endif
 
+#include <glib/gi18n.h>
 #include <glib/gerror.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkrange.h>
@@ -17,9 +18,8 @@
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkmessagedialog.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-about.h>
 #include <libgnome/gnome-help.h>
+#include <gtk/gtk.h>
 
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-control.h>
@@ -40,12 +40,13 @@ static GNOME_Media_CDDBTrackEditor track_editor = CORBA_OBJECT_NIL;
 
 static gboolean position_auto_update=TRUE;
 static gboolean position_update_ignore_event=FALSE;
+static gchar* discid_prev;
 
 static void
 maybe_close_tray (GnomeCD *gcd)
 {
 	GnomeCDRomStatus *status;
-	GError *error;
+	GError *error = NULL;
 
 	if (gnome_cdrom_get_status (gcd->cdrom, &status, &error) == FALSE) {
 		gcd_warning ("%s", error);
@@ -69,10 +70,10 @@ void
 eject_cb (GtkButton *button,
 	  GnomeCD *gcd)
 {
-	GError *error;
+	GError *error = NULL;
 
 	if (gnome_cdrom_eject (gcd->cdrom, &error) == FALSE) {
-		gcd_warning ("%s", error);
+		gcd_error ("%s", error);
 		g_error_free (error);
 	}
 
@@ -92,7 +93,7 @@ void
 play_cb (GtkButton *button,
 	 GnomeCD *gcd)
 {
-	GError *error;
+	GError *error = NULL;
 	GnomeCDRomStatus *status;
 	GnomeCDRomMSF msf;
 	AtkObject *aob;
@@ -100,7 +101,7 @@ play_cb (GtkButton *button,
 	GnomeCDRomMSF *endmsf;
 
 	if (gnome_cdrom_get_status (gcd->cdrom, &status, &error) == FALSE) {
-		gcd_warning ("%s", error);
+		gcd_error ("%s", error);
 		g_error_free (error);
 		g_free (status);
 		return;
@@ -110,7 +111,7 @@ play_cb (GtkButton *button,
 	switch (status->cd) {
 	case GNOME_CDROM_STATUS_TRAY_OPEN:
 		if (gnome_cdrom_close_tray (gcd->cdrom, &error) == FALSE) {
-			gcd_warning ("Cannot close tray: %s", error);
+			gcd_error ("Cannot close tray: %s", error);
 			g_error_free (error);
 
 			g_free (status);
@@ -139,7 +140,7 @@ play_cb (GtkButton *button,
 		break;
 
 	case GNOME_CDROM_STATUS_DRIVE_NOT_READY:
-		gcd_warning ("Drive not ready: %s", NULL);
+		gcd_error ("Drive not ready: %s", NULL);
 
 		g_free (status);
 		return;
@@ -159,7 +160,7 @@ play_cb (GtkButton *button,
 			gcd->current_image = gcd->pause_image;
 		} else { 
 			if (gnome_cdrom_pause (gcd->cdrom, &error) == FALSE) {
-				gcd_warning ("%s", error);
+				gcd_error ("%s", error);
 				g_error_free (error);
 
 				g_free (status);
@@ -190,7 +191,7 @@ play_cb (GtkButton *button,
 		}
 		if (gnome_cdrom_play (gcd->cdrom, status->track,
 				      &status->relative, end_track, endmsf, &error) == FALSE) {
-			gcd_warning ("%s", error);
+			gcd_error ("%s", error);
 			g_error_free (error);
 			
 			g_free (status);
@@ -220,7 +221,7 @@ play_cb (GtkButton *button,
 			endmsf = &msf;
 		}
 		if (gnome_cdrom_play (gcd->cdrom, 1, &msf, end_track, endmsf, &error) == FALSE) {
-			gcd_warning ("%s", error);
+			gcd_error ("%s", error);
 			g_error_free (error);
 
 			g_free (status);
@@ -237,7 +238,7 @@ play_cb (GtkButton *button,
 		break;
 
 	case GNOME_CDROM_AUDIO_ERROR:
-		gcd_warning ("Error playing CD: %s", NULL);
+		gcd_error ("Error playing CD: %s", NULL);
 		g_free (status);
 		return;
 
@@ -253,14 +254,15 @@ void
 stop_cb (GtkButton *button,
 	 GnomeCD *gcd)
 {
-	GError *error;
+	GError *error = NULL;
 
 	/* Close the tray if needed */
 	maybe_close_tray (gcd);
 
 	if (gnome_cdrom_stop (gcd->cdrom, &error) == FALSE) {
-		gcd_warning ("%s", error);
+		gcd_error ("%s", error);
 		g_error_free (error);
+		return;
 	}
 
 	if (gcd->current_image == gcd->pause_image) {
@@ -278,7 +280,7 @@ stop_cb (GtkButton *button,
 static gboolean
 ffwd_timeout_cb (gpointer data)
 {
-	GError *error;
+	GError *error = NULL;
 	GnomeCD *gcd = data;
 
 	if (gnome_cdrom_fast_forward (gcd->cdrom, &error) == FALSE) {
@@ -336,12 +338,12 @@ void
 next_cb (GtkButton *button,
 	 GnomeCD *gcd)
 {
-	GError *error;
+	GError *error = NULL;
 
 	maybe_close_tray (gcd);
 
 	if (gnome_cdrom_next (gcd->cdrom, &error) == FALSE) {
-		gcd_warning ("%s", error);
+		gcd_error ("%s", error);
 		g_error_free (error);
 	}
 
@@ -360,12 +362,12 @@ void
 back_cb (GtkButton *button,
 	 GnomeCD *gcd)
 {
-	GError *error;
+	GError *error = NULL;
 
 	maybe_close_tray (gcd);
 
 	if (gnome_cdrom_back (gcd->cdrom, &error) == FALSE) {
-		gcd_warning ("%s", error);
+		gcd_error ("%s", error);
 		g_error_free (error);
 	}
 
@@ -383,7 +385,7 @@ back_cb (GtkButton *button,
 static gboolean
 rewind_timeout_cb (gpointer data)
 {
-	GError *error;
+	GError *error = NULL;
 	GnomeCD *gcd = data;
 	
 	if (gnome_cdrom_rewind (gcd->cdrom, &error) == FALSE) {
@@ -486,6 +488,7 @@ status_ok (GnomeCD *gcd,
 	AtkObject *aob;
 	int track;
 	char *text;
+	char *current_time;
 	
 	/* Allow the track editor to work */
 	gtk_widget_set_sensitive (gcd->trackeditor_b, TRUE);
@@ -529,6 +532,7 @@ status_ok (GnomeCD *gcd,
 		text = g_strdup_printf ("%d:%02d / %d:%02d", 
 					status->relative.minute, status->relative.second,
 					status->length.minute, status->length.second);
+		current_time = g_strdup_printf ("%d:%02d",status->relative.minute, status->relative.second);
 		
 		/* update position slider */
 		if(position_auto_update && (status->length.minute!=0 ||
@@ -554,11 +558,12 @@ status_ok (GnomeCD *gcd,
 			
 		/* Update the tray icon tooltip */
 		if (gcd->disc_info != NULL) {
-			text = g_strdup_printf (_("Playing %s - %s"),
+			text = g_strdup_printf (_("Playing %s - %s\nCurrent Time: %s"),
 						gcd->disc_info->artist ? gcd->disc_info->artist : _("Unknown Artist"),
-						gcd->disc_info->title ? gcd->disc_info->title : _("Unknown Album"));
+						gcd->disc_info->title ? gcd->disc_info->title : _("Unknown Album"),
+						current_time);
 		} else {
-			text = g_strdup (_("Playing"));
+			text = g_strdup_printf (_("Playing\nCurrent Time: %s"), current_time);
 		}
 		if (gcd->tray_tips)	
 			gtk_tooltips_set_tip (gcd->tray_tips, gcd->tray, text, NULL);
@@ -593,7 +598,7 @@ status_ok (GnomeCD *gcd,
 			} else {
 				GnomeCDRomMSF msf;
 				int start_track, end_track;
-				GError *error;
+				GError *error = NULL;
 				
 				/* CD has gone to the start of the next track.
 				   Track we want to loop is track - 1 */
@@ -747,6 +752,8 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 		if (gcd->disc_info != NULL) {
 			cddb_free_disc_info (gcd->disc_info);
 			gcd->disc_info = NULL;
+			/* we need to destroy the hashtable(cddb_cache) */
+			destroy_cache_hashTable();
 		}
 		
 		gtk_widget_set_sensitive (gcd->trackeditor_b, FALSE);
@@ -800,6 +807,8 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 		if (gcd->disc_info != NULL) {
 				cddb_free_disc_info (gcd->disc_info);
 				gcd->disc_info = NULL;
+				/* we need to destroy the hashtable(cddb_cache) */
+				destroy_cache_hashTable();
 		}	
 		gtk_widget_set_sensitive (gcd->trackeditor_b, FALSE);
 
@@ -824,6 +833,8 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 		if (gcd->disc_info != NULL) {
 				cddb_free_disc_info (gcd->disc_info);
 				gcd->disc_info = NULL;
+				/* we need to destroy the hashtable(cddb_cache) */
+				destroy_cache_hashTable();
 		}	
 		gtk_widget_set_sensitive (gcd->trackeditor_b, FALSE);
 
@@ -850,6 +861,8 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 		if (gcd->disc_info != NULL) {
 			cddb_free_disc_info (gcd->disc_info);
 			gcd->disc_info = NULL;
+			/* we need to destroy the hashtable(cddb_cache) */
+			destroy_cache_hashTable();
 		}
 
 		gtk_widget_set_sensitive (gcd->trackeditor_b, FALSE);
@@ -870,23 +883,17 @@ void
 about_cb (GtkWidget *widget,
 	  gpointer data)
 {
-	static GtkWidget *about = NULL;
-	GdkPixbuf *pixbuf = NULL;
 	const char *authors[2] = {"Iain Holmes <iain@prettypeople.org>", NULL};
-	
-	if (about == NULL) {
-		pixbuf = gdk_pixbuf_new_from_file (GNOME_ICONDIR "/gnome-cd.png", NULL);
-		about = gnome_about_new (_("CD Player"), VERSION,
-					 "Copyright \xc2\xa9 2001-2002 Iain Holmes",
-					 _("A CD player for GNOME"),
-					authors, NULL, NULL, pixbuf);
-		if (pixbuf != NULL)
-			gdk_pixbuf_unref (pixbuf);
-		
-		g_signal_connect (G_OBJECT (about), "destroy",
-				  G_CALLBACK (gtk_widget_destroyed), &about);
-		gtk_widget_show (about);
-	}
+
+  	gtk_show_about_dialog (NULL,
+			       "name", _("CD Player"),
+			       "version", VERSION,
+			       "copyright", "Copyright \xc2\xa9 2001-2002 Iain Holmes",
+			       "comments", _("A CD player for GNOME"),
+			       "authors", authors,
+			       "logo-icon-name", "gnome-cd",
+		               NULL);
+
 }
 
 void
@@ -935,7 +942,7 @@ button_press_event_cb (GtkWidget *widget,
                 GnomeCD *gcd)
 {
     if ((event->button == 3) && !(event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK))) {
-        make_popup_menu (gcd, event);
+        make_popup_menu (gcd, event, FALSE);
         return TRUE;
     }
     return FALSE;
@@ -945,7 +952,7 @@ button_press_event_cb (GtkWidget *widget,
 gboolean
 popup_menu_cb (GtkWidget *widget, GnomeCD *gcd)
 {
-    make_popup_menu (gcd, NULL);
+    make_popup_menu (gcd, NULL, FALSE);
     return TRUE;
 }
 
@@ -961,6 +968,10 @@ tray_icon_clicked (GtkWidget *widget, GdkEventButton *event, GnomeCD *gcd)
 
 		return TRUE;
 	} else {
+	if (event->button == 3) {	
+		make_popup_menu(gcd, NULL, TRUE);
+		return TRUE;
+	}
 		return FALSE;
 	}
 }
@@ -1011,6 +1022,23 @@ tray_icon_expose (GtkWidget* widget, GdkEventExpose *event)
   return FALSE;
 }
 
+gboolean
+tray_icon_destroyed (GtkWidget *widget, GnomeCD *gcd)
+{
+	GdkPixbuf *pixbuf;
+	GtkWidget *box;
+
+	if (!GTK_WIDGET_VISIBLE (gcd->window)) {
+		gtk_widget_show (gcd->window);
+	}
+	gtk_widget_destroy (gcd->tray);
+	gcd->tray = NULL;
+
+	tray_icon_create (gcd);
+
+	return TRUE;
+}
+
 void
 open_preferences (GtkWidget *widget,
 		  GnomeCD *gcd)
@@ -1061,17 +1089,41 @@ void
 open_track_editor (GtkWidget *widget,
 		   GnomeCD *gcd)
 {
-	GError *error;
+	GError *error = NULL;
 	GnomeCDRomCDDBData *data;
 	CORBA_Environment ev;
 	char *discid;
-	
+	gint result = GTK_RESPONSE_YES;
+
 	if (gnome_cdrom_get_cddb_data (gcd->cdrom, &data, &error) == FALSE) {
 		gcd_warning ("gnome_cdrom_get_cddb_data returned FALSE: %s", error);
 		g_error_free (error);
 		return;
 	}
 	
+	discid = g_strdup_printf ("%08lx", (gulong) data->discid);
+
+	if (discid_prev != NULL) {
+		gint string_compare;
+		gboolean same_cd;
+		same_cd = g_str_equal(discid,discid_prev);
+		if (!same_cd) {
+			GtkWidget *dialog;
+			gchar *text = g_strdup_printf ("<b>%s</b>\n\n%s\n%s",
+            	   				    _("Contents in the CDDB have changed"),
+		                                    _("Choosing 'Yes' will delete the previous contents in CDDB."),
+                		                    _("Do you wish to continue?"));
+
+			dialog = gtk_message_dialog_new (NULL, 0,
+                		                         GTK_MESSAGE_WARNING,
+                					 GTK_BUTTONS_YES_NO,
+							 text);
+			gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
+			result = gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy(dialog);
+   		}
+	}
+	
 	CORBA_exception_init (&ev);
 	if (track_editor == CORBA_OBJECT_NIL) {
 		track_editor = bonobo_activation_activate_from_id (CDDBSLAVE_TRACK_EDITOR_IID, 0, NULL, &ev);
@@ -1098,7 +1150,11 @@ open_track_editor (GtkWidget *widget,
 
 	if (data != NULL) {
 		discid = g_strdup_printf ("%08lx", (gulong) data->discid);
-		GNOME_Media_CDDBTrackEditor_setDiscID (track_editor, discid, &ev);
+		if (result == GTK_RESPONSE_NO) 
+		GNOME_Media_CDDBTrackEditor_setDiscID (track_editor, discid_prev, &ev);
+               if (result == GTK_RESPONSE_YES){
+               GNOME_Media_CDDBTrackEditor_setDiscID (track_editor, discid, &ev);
+		discid_prev = g_strdup (discid);}
 		g_free (discid);
 	}
 	
@@ -1117,13 +1173,22 @@ volume_changed (GtkRange *range,
 		GnomeCD *gcd)
 {
 	double volume;
-	GError *error;
+	GError *error = NULL;
+	GtkTooltips *volume_level_tooltip;
+	gchar *volume_level;
+	gint scaled_volume;
 
 	volume = gtk_range_get_value (range);
 	if (gnome_cdrom_set_volume (gcd->cdrom, (int) volume, &error) == FALSE) {
 		gcd_warning ("Error setting volume: %s", error);
 		g_error_free (error);
 	}
+	/* Set the Tooltip */
+	scaled_volume = (volume/255)*100;
+	volume_level = g_strdup_printf (_("Volume %d%%"), scaled_volume);
+	gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET (gcd->slider), volume_level, NULL);
+	gconf_client_set_int (client, "/apps/gnome-cd/volume", scaled_volume, NULL);
+	g_free (volume_level);
 }
 
 void
@@ -1134,7 +1199,7 @@ position_changed (GtkRange *range,
 	int end_track;
 	GnomeCDRomStatus *status = NULL;
 	GnomeCDRomMSF msf, msf2, *endmsf;
-	GError *error;
+	GError *error = NULL;
 	gint length, pos;
 	GtkAdjustment *adj;
 	
